//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class Ax::Mail::Mail
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2003/12/14 03:13:43 $
 *   $Revision: 1.9 $
 */
//-----------------------------------------------------------------------------
#ifndef __Ax_Mail_Mail_H__
#define __Ax_Mail_Mail_H__

//-----------------------------------------------------------------------------
//     includes
//-----------------------------------------------------------------------------

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
#include <Ax/Mail/Globals.h>
#include <Ax/Mail/Export.h>

//----- local -----------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
//----- c-client --------------------------------------------------------------
//----- other includes --------------------------------------------------------

//=============================================================================
//     namespace Ax
//=============================================================================
namespace Ax
{
  //===========================================================================
  //     namespace Mail
  //===========================================================================
  namespace Mail
  {
    //=============================================================================
    //     class Mail
    //=============================================================================
    /** An instance of Mail represents one mail in a MailBox.
     *
     *  Mails are composed of a tree of one or more instances of MailPart.
     *  The method getRootPart() returns the top-most part of a mail.
     */
    class AX_MAIL_EXPORT Mail
    {
        //-------------------------------------------------------------------------
        //     friends
        //-------------------------------------------------------------------------
        /** Only MailBox should be allowed to create/destroy instances.
         */
        friend class MailBox;

      private:
        //-------------------------------------------------------------------------
        //     Mail()
        //-------------------------------------------------------------------------
        /** Constructor
         */
        Mail(MailBox *theMailBox, MESSAGECACHE *theMsgCache);

        //-------------------------------------------------------------------------
        //     ~Mail()
        //-------------------------------------------------------------------------
        /** Destructor
         */
        ~Mail();

      public:
        //-------------------------------------------------------------------------
        //     getMailBox()
        //-------------------------------------------------------------------------
        /** returns the MailBox to which this Mail belongs to.
         *
         *  @returns the MailBox to which this Mail belongs to.
         */
        MailBox *getMailBox() const;

        //-------------------------------------------------------------------------
        //     getRootPart()
        //-------------------------------------------------------------------------
        /** returns topmost MailPart of this Mail.
         *
         *  @returns topmost MailPart of this Mail.
         */
        MailPart *getRootPart();

        //-------------------------------------------------------------------------
        //     getMailNum()
        //-------------------------------------------------------------------------
        /** Returns the number (MsgNo) of this mail.
         *
         *  @returns the number (MsgNo) of this mail.
         */
        long getMailNum() const;

        //-------------------------------------------------------------------------
        //     getIsRecent()
        //-------------------------------------------------------------------------
        /** Returns whether this mail has the \c \Recent flag set.
         *
         *  @returns whether this mail has the \c \Recent flag set.
         */
        bool getIsRecent() const;

        //-------------------------------------------------------------------------
        //     getIsSeen()
        //-------------------------------------------------------------------------
        /** Returns whether this mail has the \c \Seen flag set.
         *
         *  @returns whether this mail has the \c \Seen flag set.
         */
        bool getIsSeen() const;

        //-------------------------------------------------------------------------
        //     getIsFlagged()
        //-------------------------------------------------------------------------
        /** Returns whether this mail has the \c \Flagged flag set.
         *
         *  @returns whether this mail has the \c \Flagged flag set.
         */
        bool getIsFlagged() const;

        //-------------------------------------------------------------------------
        //     getIsAnswered()
        //-------------------------------------------------------------------------
        /** Returns whether this mail has the \c \Answered flag set.
         *
         *  @returns whether this mail has the \c \Answered flag set.
         */
        bool getIsAnswered() const;

        //-------------------------------------------------------------------------
        //     getIsDeleted()
        //-------------------------------------------------------------------------
        /** Returns whether this mail has the \c \Deleted flag set.
         *
         *  @returns whether this mail has the \c \Deleted flag set.
         */
        bool getIsDeleted() const;

        //-------------------------------------------------------------------------
        //     getBodySize()
        //-------------------------------------------------------------------------
        /** Returns the size of the body of this mail in bytes.
         *
         *  @returns the size of the body of this mail in bytes.
         */
        long getBodySize() const;

        //-------------------------------------------------------------------------
        //     getUID()
        //-------------------------------------------------------------------------
        /** Returns the unique id of this mail
         *
         *  @returns the unique id of this mail
         */
        unsigned long getUID() const;

        //-------------------------------------------------------------------------
        //     getDate()
        //-------------------------------------------------------------------------
        /** Returns the date of the mail in the form:
         *  <tt>dd-mmm-yyyy hh:mm:ss +zzzz</tt>.
         *
         *  @returns the date of the mail.
         */
        std::string getDate() const;

        //-------------------------------------------------------------------------
        //     getFrom()
        //-------------------------------------------------------------------------
        /** Returns the sender of the mail.
         *
         *  This method internally calls the method \c mail_fetchfrom of
         *  \ref c_client which works as follows:
         *
         *  "If the personal name of the first address in the envelope's from
         *   item is non-NIL, it is used; otherwise a string is created by
         *   appending the mailbox of the first address, an "@", and the host of
         *   the first address."
         *
         *  @returns the sender of the mail.
         */
        std::string getFrom() const;

        //-------------------------------------------------------------------------
        //     getSubject()
        //-------------------------------------------------------------------------
        /** Returns the subject line of the mail.
         *
         *  @param   theMaxLen the maximum number of characters to return or
         *           -1 for the internal maximum length (currently limited to 4096).
         *
         *  @returns the subject line of the mail.
         */
        std::string getSubject(int theMaxLen = -1) const;

        //-------------------------------------------------------------------------
        //     getFlagsStr()
        //-------------------------------------------------------------------------
        /** Returns a displayable string of the flags of the mail.
         *
         *  The following characters are used to represent the flags:
         *
         *  \code
  aFlag[0] =  getIsSeen    ()  ? '_' : 'N';
  aFlag[1] =  getIsFlagged ()  ? 'F' : '_';
  aFlag[2] =  getIsAnswered()  ? 'A' : '_';
  aFlag[3] =  getIsDeleted ()  ? 'D' : '_';
         *  \endcode
         *
         *  @returns a displayable string of the flags of the mail.
         */
        std::string getFlagsStr() const;

        //-------------------------------------------------------------------------
        //     getBody()
        //-------------------------------------------------------------------------
        /** Returns the complete body-text of the mail in its orginal form.
         *
         *  @returns the complete body-text of the mail.
         */
        std::string getBody() const;

        //-------------------------------------------------------------------------
        //     setFlag()
        //-------------------------------------------------------------------------
        /** Set the given flag.
         *
         *  @param theFlag flag to set.
         */
        void setFlag(MailFlag theFlag);

        //-------------------------------------------------------------------------
        //     clearFlag()
        //-------------------------------------------------------------------------
        /** Clears the given flag.
         *
         *  @param theFlag flag to clear.
         */
        void clearFlag(MailFlag theFlag);

        //-------------------------------------------------------------------------
        //     getFlag()
        //-------------------------------------------------------------------------
        /** Returns whether the given flag is set.
         *
         *  @param theFlag flag to check
         *  @returns the state of the given flag.
         */
        bool getFlag(MailFlag theFlag) const;

        //-------------------------------------------------------------------------
        //     getFlatPartList()
        //-------------------------------------------------------------------------
        /** Returns a "flat" list of pointers to all MailPars of he mail.
         *
         *  The purpose of this method is to give the user of
         *  this class an easy way to iterate over all parts of a mail
         *  without the need to recursively iterate down the tree.
         *
         *  @param fWithMultiPart decides, whether the multipart-elements
         *         which are simply containers to hold other mail-parts
         *         but do not contain any data should be included in the
         *         list.
         *  @returns a list of pointers to all instances of MailPart
         */
        MailPartList getFlatPartList(bool fWithMultiPart = true);

      private:
        //-------------------------------------------------------------------------
        //     getFlagStr()
        //-------------------------------------------------------------------------
        /** Returns the IMAP-string-representation of the given flag.
         *
         *  @returns the IMAP-string-representation of the given flag.
         */
        static std::string getFlagStr(MailFlag theFlag);

        //-------------------------------------------------------------------------
        //     attributes
        //-------------------------------------------------------------------------
        /** Pointer to the mailbox this mail belongs to.
         */
        MailBox      *myMailBox ;

        /** The root part of this mail
         */
        MailPart     *myRootPart;

        /** Pointer to the \ref c_client message-cache.
         */
        MESSAGECACHE *myMsgCache;

    }; // class Mail

  }; // namespace Mail

}; // namespace Ax

#endif

