//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class Ax::Mail::MailBox
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   15.11.2003
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2003/12/19 18:54:39 $
 *   $Revision: 1.7 $
 */
//-----------------------------------------------------------------------------
#ifndef __Ax_Mail_MailBox_H__
#define __Ax_Mail_MailBox_H__

//-----------------------------------------------------------------------------
//     includes
//-----------------------------------------------------------------------------

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- ICE -------------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
#include <Ax/Mail/Globals.h>
#include <Ax/Mail/Export.h>
#include <Ax/Mail/MailBoxSetup.h>

//----- local -----------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
//----- c-client --------------------------------------------------------------
//----- other includes --------------------------------------------------------

//=============================================================================
//     namespace Ax
//=============================================================================
namespace Ax
{
  //===========================================================================
  //     namespace Mail
  //===========================================================================
  namespace Mail
  {
    //=============================================================================
    //     class MailBox
    //=============================================================================
    /** An instance of MailBox is used to access a mail account and to retrieve
     *  mails from it.
     */
    class AX_MAIL_EXPORT MailBox
    {
      public:
        //-------------------------------------------------------------------------
        //     MailBox()
        //-------------------------------------------------------------------------
        /** Constructor
         *
         *  Note: MailBox doesn't copy the contents of theSetup, instead it holds
         *  a pointer the the given setup. Therefore theSetup must exists as long
         *  as an instance of MailBox exist.
         *
         *  \todo think about the memory managmnt of theSetup.
         *
         *  @param theSetup contains the settings for a mailbox.
         */
        MailBox(MailBoxSetup *theSetup);

        //-------------------------------------------------------------------------
        //     ~MailBox()
        //-------------------------------------------------------------------------
        /** Destructor
         *
         *  Destroys all instances of Mail and closes the connection if isOpen().
         */
        virtual ~MailBox();

        //-------------------------------------------------------------------------
        //     stream()
        //-------------------------------------------------------------------------
        /** Returns a pointer to the \c c-client stream type used for the connection
         *  to the mailbox.
         *
         *  @returns a pointer to the \c c-client stream used for the connection.
         */
        MAILSTREAM *stream() const;

        //-------------------------------------------------------------------------
        //     getMail()
        //-------------------------------------------------------------------------
        /** Returns the instance with the given mail number.
         *
         *  The mail number has to be in the range of 1..getCountMails().
         *
         *  @param   theMailNum is the number of the mail [1..getCountMails()]
         *  @returns a pointer to the Mail-instance with the given
         *           mail number or 0 if theMailNum is out of range or if the
         *           mailbox was not able to fetch the mail.
         */
        Mail *getMail(long theMailNum);

        //-------------------------------------------------------------------------
        //     getSetup()
        //-------------------------------------------------------------------------
        /** Returns a pointer of the setup of this mailbox.
         *
         *  @returns a pointer of the setup of this mailbox.
         */
        const MailBoxSetup *getSetup() const;

        //-------------------------------------------------------------------------
        //     getAccountName()
        //-------------------------------------------------------------------------
        /** Returns the symbolic name of this mailbox (from the setup).
         *
         *  @returns the symbolic name of this mailbox.
         */
        char *getAccountName() const;

        //-------------------------------------------------------------------------
        //     getLoginName()
        //-------------------------------------------------------------------------
        /** Returns the login name to the mailbox (from the setup).
         *
         *  @returns the login name to the mailbox.
         */
        char *getLoginName() const;

        //-------------------------------------------------------------------------
        //     getLoginPass()
        //-------------------------------------------------------------------------
        /** Returns the login password to the mailbox (from the setup).
         *
         *  @returns the login password to the mailbox.
         */
        char *getLoginPass() const;

        //-------------------------------------------------------------------------
        //     getMailBox()
        //-------------------------------------------------------------------------
        /** Returns the mailbox-string to access the mailbox (from the setup).
         *
         *  @returns the mailbox-string to access the mailbox.
         */
        char *getMailBox() const;

        //-------------------------------------------------------------------------
        //     getCountMails()
        //-------------------------------------------------------------------------
        /** Returns the total number of mails.
         *
         *  @returns the total number of mails.
         */
        long getCountMails() const;

        //-------------------------------------------------------------------------
        //     getCountUnseen()
        //-------------------------------------------------------------------------
        /** Returns the number of unseen (not \Seen) mails.
         *
         *  @returns the number of unseen (not \Seen) mails.
         */
        long getCountUnseen() const;

        //-------------------------------------------------------------------------
        //     getCountRecent()
        //-------------------------------------------------------------------------
        /** Returns the number of recent (\Recent) mails.
         *
         *  @returns the number of recent (\Recent) mails.
         */
        long getCountRecent() const;

        //-------------------------------------------------------------------------
        //     getLastLogMsg()
        //-------------------------------------------------------------------------
        /** Returns the last message text returned from \c c-client.
         *
         *  @returns the last message text returned from \c c-client.
         */
        std::string getLastLogMsg () const;

        //=========================================================================
        //     commands to the mail server
        //=========================================================================

        //-------------------------------------------------------------------------
        //     requestStatus()
        //-------------------------------------------------------------------------
        /** Initiates a status-request to get information about the mailbox.
         *
         *  This method must be called before any calls to getCountMails(),
         *  getCountUnseen() and getCountRecent().
         *
         */
        bool requestStatus();

        //-------------------------------------------------------------------------
        //     hasStatus()
        //-------------------------------------------------------------------------
        /** Returns if we have the state of the mailbox either by a call
         *  to requestState() or by a call to open().
         *
         *  @returns true if the last requestStatus() or open() succeeded.
         */
        bool hasStatus() const;

        //-------------------------------------------------------------------------
        //     open()
        //-------------------------------------------------------------------------
        /** Opens the connection to the mailbox.
         *
         *  This method must be called, before getMail() may be called.
         */
        bool open();

        //-------------------------------------------------------------------------
        //     isOpen()
        //-------------------------------------------------------------------------
        /** Returns true if the connection to the mailbox is established.
         *
         *  @returns true if the connection to the mailbox is established.
         */
        bool isOpen() const { return myStream != NIL; }

        //-------------------------------------------------------------------------
        //     close()
        //-------------------------------------------------------------------------
        /** Close the connection to the mailbox.
         */
        bool close();

        //=========================================================================
        //     process messages returned from the mail server
        //=========================================================================

        //-------------------------------------------------------------------------
        //     processMailStatus()
        //-------------------------------------------------------------------------
        /** Callback for demultiplexer of \c c-client: process the answer of
         *  requestStatus().
         */
        void processMailStatus(MAILSTATUS *theStatus);

        //-------------------------------------------------------------------------
        //     processLogMsg()
        //-------------------------------------------------------------------------
        /** Callback for demultiplexer of \c c-client: process log messages.
         */
        void processLogMsg(const char *theMsg, long theErrorFlag);

        //=========================================================================
        //     some helper methods
        //=========================================================================
        //-------------------------------------------------------------------------
        //     getNext()
        //-------------------------------------------------------------------------
        /** Returns the next mail after the given mail or 0 if not possible.
         *
         *  Returns 0 if the mailbox doesn't contain any mails, the given Mail
         *  isn't within this mailbox or the given mail is the last mail in the
         *  mailbox.
         *
         *  Returns the first mail if the given mail is 0 and there is at least
         *  one mail in the mailbox.
         *
         *  @returns the next mail after the given or the first mail if
         *           the given mail is 0.
         */
        Mail *getNext(Mail *theMail = 0);

        //-------------------------------------------------------------------------
        //     getPrev()
        //-------------------------------------------------------------------------
        /** Returns the previous mail before the given mail or 0 if not possible.
         *
         *  Returns 0 if the mailbox doesn't contain any mails, the given Mail
         *  isn't within this mailbox or the given mail is the first mail in the
         *  mailbox.
         *
         *  Returns the last mail if the given mail is 0 and there is at least
         *  one mail in the mailbox.
         *
         *  @returns the previous mail before the given or the last mail if
         *           the given mail is 0.
         */
        Mail *getPrev(Mail *theMail = 0);

        //-------------------------------------------------------------------------
        //     getNextNew()
        //-------------------------------------------------------------------------
        /** Returns the next new mail after the given mail or 0 if not possible.
         *
         *  Returns 0 if the mailbox doesn't contain any new mails or there
         *  is no new mail after the given Mail.
         *
         *  Returns the first new mail if the given mail is 0 or the given mail
         *  is not in the mailbox.
         *
         *  @returns the next new mail after the given or the first mail if
         *           the given mail is 0.
         */
        Mail *getNextNew(Mail *theMail = 0);

        //-------------------------------------------------------------------------
        //     getPrevNew()
        //-------------------------------------------------------------------------
        /** Returns the previous new mail before the given mail or 0 if not possible.
         *
         *  Returns 0 if the mailbox doesn't contain any new mails or there
         *  is no new mail before the given Mail.
         *
         *  Returns the last new mail if the given mail is 0 or the given mail
         *  is not in the mailbox.
         *
         *  @returns the previous new mail before the given or the last new mail if
         *           the given mail is 0.
         */
        Mail *getPrevNew(Mail *theMail = 0);

        //-------------------------------------------------------------------------
        //     getSurroundings()
        //-------------------------------------------------------------------------
        /** Returns the mails surrounding
         *
         */
        void getSurroundings( Mail * theMail    = 0
                            , Mail **thePrev    = 0
                            , Mail **theNext    = 0
                            , Mail **thePrevNew = 0
                            , Mail **theNextNew = 0
                            );

      private:
        //-------------------------------------------------------------------------
        //     attributes
        //-------------------------------------------------------------------------
        /** Pointer to the setup for this mailbox.
         */
        MailBoxSetup    *mySetup;

        /** Pointer to the \c c-client mailstream used for the connection to the
         *  mailbox.
         */
        MAILSTREAM      *myStream;

        /** true if status of mailbox is known.
         */
        bool             myHasStatus;

        /** the total number of mails
         */
        long             myCountMails;

        /** the number of new (not \Seen) mails
         */
        long             myCountUnseen;

        /** the number of recent (\Recent) mails
         */
        long             myCountRecent;

        /** the last message we received from \c c-client
         */
        std::string      myLastLogMsg;

        /** a vector of pointers to Mail instances
         */
        MailVector       myMails;

    }; // class MailBox

  }; // namespace Mail

}; // namespace Ax

#endif

